#!/bin/sh
# the next line restarts using tclsh \
    exec tclsh "$0" "$@"

# jpegarch.tcl --
# This file 

# $Id: jpegarch.tcl,v 1.2 2003/09/10 19:40:45 davidw Exp $

namespace eval jpegarch {
    # Files and their sizes.
    array set files {}
    # Order to write the files in.
    set order {}
}

proc lremove {list string} {
    set s [lsearch $list $string]
    if {$s >= 0} {
	set list [lreplace $list $s $s]
    }
    return $list
}

# jpegarch::AddFile --
#
#	Add a file (as well as its size) to the list of files to be
#	archived.
#
# Arguments:
#	filename.

proc jpegarch::AddFile {filename} {
    variable files
    variable order
    set files($filename) [file size $filename]
    lappend order $filename
}


# jpegarch::CreateArchive --
#
# 	Create the JPEG archive.  Archive format is an index table of
# 	offsets stored as little-endian integers, which should be
# 	large enough to hold anything on a floppy disk.  The index
# 	table is, in turn, headed by a little endian integer
# 	indicating the number of entries in the table.
#
# Arguments:
#
#	filename - the archive name.

proc jpegarch::CreateArchive {filename} {
    variable files
    variable order

    set fl [open $filename w]
    fconfigure $fl -encoding binary
    fconfigure $fl -translation binary

    # Offset in archive file.
    set offset 0
    # Length of index table, in bytes.  Add 2 - one for the length of
    # the table, one for the last entry indicating the end location of
    # the final jpeg.
    set tablelength [expr {([llength $order] + 2) * 4}]
    incr offset $tablelength
    puts -nonewline $fl [binary format i [llength $order]]

    # Create the index table.
    foreach jpg $order {
	puts "Adding $jpg to the archive at $offset"
	puts -nonewline $fl [binary format i $offset]
	incr offset $files($jpg)
    }
    puts -nonewline $fl [binary format i $offset]
    puts "Last image ends at $offset"

    # Now copy the files into the archive.
    foreach jpg $order {
	set jpgfl [open $jpg]
	fconfigure $jpgfl -encoding binary
	fconfigure $jpgfl -translation binary
	puts -nonewline $fl [read $jpgfl]
	close $jpgfl
    }

    close $fl
}


# jpegarch::GetOrder --
#
#	Get the list of files to be archived.
#
# Results:
#	A list of files.

proc jpegarch::GetOrder {} {
    variable order
    return $order
}


# jpegarch::SetOrder --
#
#	Set the order of files to be archived.
#
# Arguments:
#	neworder - a list of all the files to be archived.  These
#	should be actual files!

proc jpegarch::SetOrder {neworder} {
    variable order
    set order $neworder
}

# If we are being called from the command line.
if { $argv0 == [info script] } {
    jpegarch::AddFile splash.jpg
    # Add files on the command line.
    foreach fl $argv {
	jpegarch::AddFile $fl
    }
    jpegarch::CreateArchive archive.bin
}

