#!/bin/sh
# the next line restarts using tclsh \
    exec tclsh "$0" "$@"

package require md5

namespace eval md5check {
    set imagesize 0
}


# md5check::read2mem --
#
#	Read a file without translation or encoding into memory and
#	return it.
#
# Arguments:
#	filename
#	size - size of file - if 0, fetch the whole file.
#
# Side Effects:
#	None.
#
# Results:
#	Data from file read.

proc md5check::read2mem {filename {size 0}} {
    set fl [open $filename r]
    fconfigure $fl -translation binary
    fconfigure $fl -encoding binary
    if { $size != 0 } {
	set data [read $fl $size]
    } else {
	set data [read $fl]
    }
    close $fl
    return $data
}


# md5check::write2floppy --
#
#	Takes a file and writes it to the floppy device.  You can't
#	use 'file copy' because that overwrites the device node!
#
# Arguments:
#	filename - file to write.
#	device   - device to write to (like /dev/fd0 on Linux).
#
# Side Effects:
#	None.
#
# Results:
#	None.

if { $tcl_platform(platform) == "unix" } {
    proc md5check::write2floppy {filename device} {
	set ifl [open $filename]
	fconfigure $ifl -translation binary
	fconfigure $ifl -encoding binary
	set ofl [open $device w]
	fconfigure $ofl -translation binary
	fconfigure $ofl -encoding binary
	puts -nonewline $ofl [read $ifl]
	close $ifl
	close $ofl
    }
}


# md5check::readfromfloppy --
#
#	Copy data from floppy into file.
#
# Arguments:
#	device - floppy device.
#	filename - destination.
#	size - how many bytes to copy.
#
# Side Effects:
#	None.
#
# Results:
#	None.

proc md5check::readfromfloppy {device filename size} {
    set ofl [open $filename w]
    fconfigure $ofl -translation binary
    fconfigure $ofl -encoding binary
    puts -nonewline $ofl [read2mem $device $size]
    close $ofl
}

proc md5check::main {filename device} {
    set woops "You might consider trying again with a different floppy disk."
    set gooddata [read2mem $filename]
    set goodmsg [md5::md5 $gooddata]
    unset gooddata
    set tmpfile [file join / tmp tmp.[clock clicks]]
    if { [catch {
	write2floppy $filename $device
	readfromfloppy $device $tmpfile [file size $filename]
    } err] } {
	puts "Problem with floppy: $err"
	puts $woops
	return
    }
    set checkdata [read2mem $tmpfile]
    set checkmsg [md5::md5 $checkdata]
    unset checkdata
    file delete $tmpfile
    puts "Image file has md5: $goodmsg"
    puts "Floppy file has   : $checkmsg"
    if { "$goodmsg" == "$checkmsg" } {
	puts "md5sum's match, floppy is good."
    } else {
	puts "md5 sums do *not* match - looks like there was a problem!"
	puts $woops
    }
}

md5check::main [lindex $argv 0] [lindex $argv 1]
